"""
Dashboard commands for viewing tasks, entities, and status
"""
import click
from goldfish_backend.core.database import create_db_and_tables, engine
from goldfish_backend.models.note import Note
from goldfish_backend.models.person import Person
from goldfish_backend.models.project import Project
from goldfish_backend.models.suggested_entity import SuggestedEntity
from goldfish_backend.models.topic import Topic
from goldfish_backend.models.user import User
from rich.columns import Columns
from rich.console import Console
from rich.panel import Panel
from rich.table import Table
from sqlmodel import Session, desc, func, select

from .main import dashboard

console = Console()


@dashboard.command()
@click.option('--user-id', default=1, help='User ID (default: 1)')
def status(user_id):
    """
    Show overall dashboard status

    Example:
        goldfish dashboard status
    """
    create_db_and_tables()

    with Session(engine) as db:
        user = db.get(User, user_id)
        if not user:
            console.print(f"❌ User with ID {user_id} not found. Run: goldfish config setup")
            return

        # Welcome message
        console.print(f"🐠 Welcome back, {user.full_name}!")

        # Get statistics
        stats = _get_user_stats(db, user_id)

        # Create dashboard panels
        panels = []

        # Capture stats
        capture_stats = f"""
📝 Capture Activity:
• Notes captured: {stats['notes']}
• Total characters: {stats['total_chars']:,}
• Latest capture: {stats['latest_note']}
        """
        panels.append(Panel(capture_stats.strip(), title="Capture", border_style="green"))

        # AI suggestions
        ai_stats = f"""
🤖 AI Processing:
• Pending suggestions: {stats['pending_suggestions']}
• Confirmed entities: {stats['confirmed_entities']}
• Accuracy rate: {stats['accuracy_rate']:.1f}%
        """
        panels.append(Panel(ai_stats.strip(), title="AI Status", border_style="blue"))

        # Entities
        entity_stats = f"""
📊 Knowledge Base:
• People: {stats['people']}
• Projects: {stats['projects']}
• Topics: {stats['topics']}
        """
        panels.append(Panel(entity_stats.strip(), title="Entities", border_style="yellow"))

        # Show panels in columns
        console.print(Columns(panels, equal=True))

        # Show suggestions status
        if stats['pending_suggestions'] > 0:
            console.print(f"\n🔔 You have {stats['pending_suggestions']} pending suggestions")
            console.print("📋 Run: [cyan]goldfish suggestions pending[/cyan] to review")
        else:
            console.print("\n✅ All suggestions reviewed! Great work!")

        # Quick actions
        actions = """
🚀 Quick Actions:
• Capture text: goldfish capture quick "your text"
• Review suggestions: goldfish suggestions pending
• View entities: goldfish dashboard entities
• File help: goldfish --help
        """

        console.print(Panel(actions.strip(), title="Next Steps", border_style="cyan"))


@dashboard.command()
@click.option('--user-id', default=1, help='User ID (default: 1)')
@click.option('--type', 'entity_type', help='Filter by entity type (people, projects, topics)')
def entities(user_id, entity_type):
    """
    Show all entities in the knowledge base

    Example:
        goldfish dashboard entities --type people
    """
    create_db_and_tables()

    with Session(engine) as db:
        user = db.get(User, user_id)
        if not user:
            console.print(f"❌ User with ID {user_id} not found")
            return

        if entity_type:
            _show_entities_by_type(db, user_id, entity_type)
        else:
            _show_all_entities(db, user_id)


@dashboard.command()
@click.option('--user-id', default=1, help='User ID (default: 1)')
@click.option('--limit', default=10, help='Number of notes to show (default: 10)')
def notes(user_id, limit):
    """
    Show recent notes

    Example:
        goldfish dashboard notes --limit 5
    """
    create_db_and_tables()

    with Session(engine) as db:
        user = db.get(User, user_id)
        if not user:
            console.print(f"❌ User with ID {user_id} not found")
            return

        # Get recent notes
        statement = select(Note).where(
            Note.user_id == user_id,
            not Note.is_deleted
        ).order_by(Note.created_at.desc()).limit(limit)

        notes = db.exec(statement).all()

        if not notes:
            console.print("📝 No notes found")
            console.print("💡 Try: goldfish capture quick 'Your first note'")
            return

        table = Table(title=f"Recent Notes (Latest {len(notes)})", show_header=True)
        table.add_column("ID", style="cyan", width=6)
        table.add_column("Content", style="white", width=50)
        table.add_column("Created", style="dim", width=20)
        table.add_column("Source", style="green", width=15)

        for note in notes:
            content_preview = note.content[:47] + "..." if len(note.content) > 50 else note.content
            source = note.processing_metadata.get('source', 'unknown') if note.processing_metadata else 'unknown'
            created = note.created_at.strftime('%Y-%m-%d %H:%M')

            table.add_row(
                str(note.id),
                content_preview,
                created,
                source
            )

        console.print(table)


def _get_user_stats(db: Session, user_id: int) -> dict:
    """Get user statistics for dashboard"""

    # Notes
    note_count = db.exec(select(func.count()).select_from(Note).where(Note.user_id == user_id)).one()

    # Characters
    total_chars = db.exec(select(func.sum(func.length(Note.content))).where(Note.user_id == user_id)).one() or 0

    # Latest note
    latest_note = db.exec(select(Note).where(Note.user_id == user_id).order_by(desc(Note.created_at)).limit(1)).first()
    latest_str = latest_note.created_at.strftime('%Y-%m-%d') if latest_note else 'Never'

    # Suggestions
    pending_suggestions = db.exec(select(func.count()).select_from(SuggestedEntity).where(
        SuggestedEntity.user_id == user_id,
        SuggestedEntity.status == "pending"
    )).one()

    confirmed_entities = db.exec(select(func.count()).select_from(SuggestedEntity).where(
        SuggestedEntity.user_id == user_id,
        SuggestedEntity.status == "confirmed"
    )).one()

    total_suggestions = db.exec(select(func.count()).select_from(SuggestedEntity).where(
        SuggestedEntity.user_id == user_id,
        (SuggestedEntity.status == "confirmed") | (SuggestedEntity.status == "rejected")
    )).one()

    accuracy_rate = (confirmed_entities / total_suggestions * 100) if total_suggestions > 0 else 0

    # Entities
    people_count = db.exec(select(func.count()).select_from(Person).where(Person.user_id == user_id)).one()
    projects_count = db.exec(select(func.count()).select_from(Project).where(Project.user_id == user_id)).one()
    topics_count = db.exec(select(func.count()).select_from(Topic).where(Topic.user_id == user_id)).one()

    return {
        'notes': note_count,
        'total_chars': total_chars,
        'latest_note': latest_str,
        'pending_suggestions': pending_suggestions,
        'confirmed_entities': confirmed_entities,
        'accuracy_rate': accuracy_rate,
        'people': people_count,
        'projects': projects_count,
        'topics': topics_count,
    }


def _show_all_entities(db: Session, user_id: int):
    """Show all entities grouped by type"""

    # People
    people = db.exec(select(Person).where(Person.user_id == user_id, not Person.is_deleted)).all()
    if people:
        table = Table(title="👥 People", show_header=True)
        table.add_column("ID", width=6)
        table.add_column("Name", width=25)
        table.add_column("Importance", width=12)
        table.add_column("Email", width=25)

        for person in people:
            table.add_row(
                str(person.id),
                person.name,
                f"{person.importance_score:.1f}/10",
                person.email or "Not set"
            )
        console.print(table)

    # Projects
    projects = db.exec(select(Project).where(Project.user_id == user_id, not Project.is_deleted)).all()
    if projects:
        table = Table(title="📁 Projects", show_header=True)
        table.add_column("ID", width=6)
        table.add_column("Name", width=25)
        table.add_column("Status", width=12)
        table.add_column("Priority", width=12)
        table.add_column("Deadline", width=15)

        for project in projects:
            deadline_str = project.deadline.strftime('%Y-%m-%d') if project.deadline else "Not set"
            status_color = "green" if project.status == "active" else "yellow"

            table.add_row(
                str(project.id),
                project.name,
                f"[{status_color}]{project.status}[/{status_color}]",
                f"{project.priority_score:.1f}/10",
                deadline_str
            )
        console.print(table)

    # Topics
    topics = db.exec(select(Topic).where(Topic.user_id == user_id, not Topic.is_deleted)).all()
    if topics:
        table = Table(title="🧠 Topics", show_header=True)
        table.add_column("ID", width=6)
        table.add_column("Name", width=25)
        table.add_column("Category", width=15)
        table.add_column("Research Score", width=15)

        for topic in topics:
            table.add_row(
                str(topic.id),
                topic.name,
                topic.category or "Uncategorized",
                f"{topic.research_score:.1f}/10"
            )
        console.print(table)

    if not (people or projects or topics):
        console.print("📋 No entities found yet")
        console.print("💡 Capture some text to get started: goldfish capture quick 'Your text here'")


def _show_entities_by_type(db: Session, user_id: int, entity_type: str):
    """Show entities of a specific type"""

    if entity_type == "people":
        people_entities = db.exec(select(Person).where(Person.user_id == user_id, not Person.is_deleted)).all()
        if people_entities:
            table = Table(title="👥 People", show_header=True)
            table.add_column("ID", width=6)
            table.add_column("Name", width=25)
            table.add_column("Importance", width=12)
            table.add_column("Email", width=25)
            table.add_column("Bio", width=30)

            for person in people_entities:
                table.add_row(
                    str(person.id),
                    person.name,
                    f"{person.importance_score:.1f}/10",
                    person.email or "Not set",
                    person.bio[:27] + "..." if person.bio and len(person.bio) > 30 else person.bio or "Not set"
                )
            console.print(table)

        if not people_entities:
            console.print(f"📋 No {entity_type} found yet")
            console.print("💡 Capture some text to get started: goldfish capture quick 'Your text here'")

    elif entity_type == "projects":
        project_entities = db.exec(select(Project).where(Project.user_id == user_id, not Project.is_deleted)).all()
        if project_entities:
            table = Table(title="📁 Projects", show_header=True)
            table.add_column("ID", width=6)
            table.add_column("Name", width=25)
            table.add_column("Status", width=12)
            table.add_column("Priority", width=12)
            table.add_column("Deadline", width=15)
            table.add_column("Description", width=30)

            for project in project_entities:
                deadline_str = project.deadline.strftime('%Y-%m-%d') if project.deadline else "Not set"
                status_color = "green" if project.status == "active" else "yellow"
                desc = project.description[:27] + "..." if project.description and len(project.description) > 30 else project.description or "Not set"

                table.add_row(
                    str(project.id),
                    project.name,
                    f"[{status_color}]{project.status}[/{status_color}]",
                    f"{project.priority_score:.1f}/10",
                    deadline_str,
                    desc
                )
            console.print(table)

        if not project_entities:
            console.print(f"📋 No {entity_type} found yet")
            console.print("💡 Capture some text to get started: goldfish capture quick 'Your text here'")

    elif entity_type == "topics":
        topic_entities = db.exec(select(Topic).where(Topic.user_id == user_id, not Topic.is_deleted)).all()
        if topic_entities:
            table = Table(title="🧠 Topics", show_header=True)
            table.add_column("ID", width=6)
            table.add_column("Name", width=25)
            table.add_column("Category", width=15)
            table.add_column("Research Score", width=15)
            table.add_column("Description", width=30)

            for topic in topic_entities:
                desc = topic.description[:27] + "..." if topic.description and len(topic.description) > 30 else topic.description or "Not set"

                table.add_row(
                    str(topic.id),
                    topic.name,
                    topic.category or "Uncategorized",
                    f"{topic.research_score:.1f}/10",
                    desc
                )
            console.print(table)

        if not topic_entities:
            console.print(f"📋 No {entity_type} found yet")
            console.print("💡 Capture some text to get started: goldfish capture quick 'Your text here'")

    else:
        console.print(f"❌ Unknown entity type: {entity_type}")
        console.print("💡 Available types: people, projects, topics")
